<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
namespace DistributeModule\Controller;

use App\Lib\Traits\PluginManagerTableTrait;
use App\Model\Entity\Host;
use App\Model\Entity\Service;
use App\Model\Table\ContainersTable;
use App\Model\Table\HostsTable;
use App\Model\Table\ServicesTable;
use App\Model\Table\SystemdowntimesTable;
use Cake\Http\Exception\MethodNotAllowedException;
use Cake\Http\Exception\NotFoundException;
use Cake\ORM\TableRegistry;
use Cake\Utility\Hash;
use DistributeModule\Filter\SystemdowntimesFilter;
use DistributeModule\Lib\SatSyncMethods;
use DistributeModule\Model\Entity\Satellite;
use DistributeModule\Model\Table\SatellitesTable;
use itnovum\openITCOCKPIT\Core\AngularJS\Api;
use itnovum\openITCOCKPIT\Core\AngularJS\Request\AngularRequest;
use itnovum\openITCOCKPIT\Core\HostConditions;
use itnovum\openITCOCKPIT\Core\System\Gearman;
use itnovum\openITCOCKPIT\Core\SystemdowntimesConditions;
use itnovum\openITCOCKPIT\Core\ValueObjects\User;
use itnovum\openITCOCKPIT\Core\Views\ContainerPermissions;
use itnovum\openITCOCKPIT\Core\Views\Systemdowntime;
use itnovum\openITCOCKPIT\Database\PaginateOMat;
use itnovum\openITCOCKPIT\Filter\SatelliteFilter;

class SatellitesController extends AppController {

    use PluginManagerTableTrait;

    public function index() {
        if (!$this->isApiRequest()) {
            /** @var User $User */
            $User = new User($this->getUser());
            $this->set('username', $User->getFullName());
            //Only ship HTML template
            return;
        }

        /** @var $SatellitesTable SatellitesTable */
        $SatellitesTable = TableRegistry::getTableLocator()->get('DistributeModule.Satellites');
        /** @var $ContainersTable ContainersTable */
        $ContainersTable = TableRegistry::getTableLocator()->get('Containers');

        $SatelliteFilter = new SatelliteFilter($this->request);

        $PaginateOMat = new PaginateOMat($this, $this->isScrollRequest(), $SatelliteFilter->getPage());

        $MY_RIGHTS = [];
        if ($this->hasRootPrivileges === false) {
            $MY_RIGHTS = $this->MY_RIGHTS;
        }

        $SatSyncMethods = new SatSyncMethods();
        $syncMethodsArray = $SatSyncMethods->getSyncMethods();
        $all_satellites_tmp = $SatellitesTable->getSatellitesIndex($SatelliteFilter, $PaginateOMat, $MY_RIGHTS);
        $all_satellites = [];
        foreach ($all_satellites_tmp as $satellite) {
            /** @var Satellite $satellite */
            $satellite = $satellite->toArray();
            $satellite['container'] = '/' . $ContainersTable->treePath($satellite['container_id']);
            $satellite['sync_method_name'] = $syncMethodsArray[$satellite['sync_method']];
            if ($this->hasRootPrivileges) {
                $satellite['allow_edit'] = true;
            } else {
                $satellite['allow_edit'] = $this->isWritableContainer($satellite['container_id']);
            }
            $all_satellites[] = $satellite;
        }

        $this->set('all_satellites', $all_satellites);
        $this->viewBuilder()->setOption('serialize', ['all_satellites']);
    }

    public function add() {
        if (!$this->isApiRequest()) {
            //Only ship template for AngularJs
            return;
        }

        if ($this->request->is('post') || $this->request->is('put')) {
            /** @var $SatellitesTable SatellitesTable */
            $SatellitesTable = TableRegistry::getTableLocator()->get('DistributeModule.Satellites');

            $satellite = $SatellitesTable->newEntity($this->request->getData('Satellite', []));
            $SatellitesTable->save($satellite);
            if ($satellite->hasErrors()) {
                $this->response = $this->response->withStatus(400);
                $this->set('error', $satellite->getErrors());
                $this->viewBuilder()->setOption('serialize', ['error']);
                return;
            }

            $this->set('satellite', $satellite);
            $this->viewBuilder()->setOption('serialize', ['satellite']);
        }
    }

    /**
     * @param int|null $id
     */
    public function edit($id = null) {
        if (!$this->isApiRequest()) {
            //Only ship template for AngularJs
            return;
        }

        /** @var $SatellitesTable SatellitesTable */
        $SatellitesTable = TableRegistry::getTableLocator()->get('DistributeModule.Satellites');

        if (!$SatellitesTable->existsById($id)) {
            throw new NotFoundException(__('Satellite not found'));
        }

        $satellite = $SatellitesTable->get($id);

        if (!$this->allowedByContainerId($satellite->get('container_id'))) {
            $this->render403();
            return;
        }

        if ($this->request->is('get') && $this->isAngularJsRequest()) {
            //Return user information
            $this->set('satellite', $satellite);
            $this->viewBuilder()->setOption('serialize', ['satellite']);
            return;
        }

        if ($this->request->is('post') || $this->request->is('put')) {
            $satellite->setAccess('id', false);

            $satellite = $SatellitesTable->patchEntity($satellite, $this->request->getData('Satellite', []));
            $SatellitesTable->save($satellite);
            if ($satellite->hasErrors()) {
                $this->response = $this->response->withStatus(400);
                $this->set('error', $satellite->getErrors());
                $this->viewBuilder()->setOption('serialize', ['error']);
                return;
            }

            $this->set('satellite', $satellite);
            $this->viewBuilder()->setOption('serialize', ['satellite']);
        }
    }

    /**
     * @param int|null $id
     */
    public function usedBy($id = null) {
        if (!$this->isAngularJsRequest()) {
            //Only ship template
            return;
        }

        /** @var $SatellitesTable SatellitesTable */
        $SatellitesTable = TableRegistry::getTableLocator()->get('DistributeModule.Satellites');
        /** @var $HostsTable HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');

        if (!$SatellitesTable->existsById($id)) {
            throw new NotFoundException(__('Satellite not found'));
        }

        $satellite = $SatellitesTable->get($id);

        if (!$this->allowedByContainerId($satellite->get('container_id'), false)) {
            $this->render403();
            return;
        }

        $HostCondition = new HostConditions();
        $HostCondition->setIncludeDisabled(false);
        $HostCondition->setContainerIds($this->MY_RIGHTS);
        $HostCondition->setSatelliteId($id);

        $hosts = $HostsTable->getHostsByHostConditions($HostCondition);

        $all_hosts = [];

        foreach ($hosts as $host) {
            $Host = new \itnovum\openITCOCKPIT\Core\Views\Host($host['Host']);

            if ($this->hasRootPrivileges) {
                $allowEdit = true;
            } else {
                $ContainerPermissions = new ContainerPermissions($this->MY_RIGHTS_LEVEL, $Host->getContainerIds());
                $allowEdit = $ContainerPermissions->hasPermission();
            }

            $host = $Host->toArray();
            $host['allow_edit'] = $allowEdit;

            $all_hosts[] = $host;
        }

        $this->set('satellite', $satellite);
        $this->set('all_hosts', $all_hosts);
        $this->viewBuilder()->setOption('serialize', ['satellite', 'all_hosts']);
    }

    public function status() {
        if (!$this->isAngularJsRequest()) {
            //Only ship HTML Template
            return;
        }

        /** @var $SatellitesTable SatellitesTable */
        $SatellitesTable = TableRegistry::getTableLocator()->get('DistributeModule.Satellites');
        /** @var $ContainersTable ContainersTable */
        $ContainersTable = TableRegistry::getTableLocator()->get('Containers');

        $SatelliteFilter = new SatelliteFilter($this->request);

        $PaginateOMat = new PaginateOMat($this, $this->isScrollRequest(), $SatelliteFilter->getPage());

        $MY_RIGHTS = [];
        if ($this->hasRootPrivileges === false) {
            $MY_RIGHTS = $this->MY_RIGHTS;
        }

        $User = new User($this->getUser());
        $UserTime = $User->getUserTime();

        $SatSyncMethods = new SatSyncMethods();
        $syncMethodsArray = $SatSyncMethods->getSyncMethods();
        $all_satellites_tmp = $SatellitesTable->getSatellitesStatus($SatelliteFilter, $PaginateOMat, $MY_RIGHTS);
        $all_satellites = [];
        foreach ($all_satellites_tmp as $satellite) {
            /** @var Satellite $satellite */
            $satellite = $satellite->toAngularArray($UserTime);
            $satellite['container'] = '/' . $ContainersTable->treePath($satellite['container_id']);
            $satellite['sync_method_name'] = $syncMethodsArray[$satellite['sync_method']];
            if ($this->hasRootPrivileges) {
                $satellite['allow_edit'] = true;
            } else {
                $satellite['allow_edit'] = $this->isWritableContainer($satellite['container_id']);
            }
            $all_satellites[] = $satellite;
        }

        $this->set('all_satellites', $all_satellites);
        $this->viewBuilder()->setOption('serialize', ['all_satellites']);
    }

    /**
     * @param int $id
     */
    public function delete($id) {
        if (!$this->request->is('post')) {
            throw new MethodNotAllowedException();
        }

        /** @var SatellitesTable $SatellitesTable */
        $SatellitesTable = TableRegistry::getTableLocator()->get('DistributeModule.Satellites');
        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
        /** @var ServicesTable $ServicesTable */
        $ServicesTable = TableRegistry::getTableLocator()->get('Services');

        if (!$SatellitesTable->existsById($id)) {
            throw new NotFoundException(__('Satellite not found'));
        }

        $satellite = $SatellitesTable->get($id);

        if (!$this->allowedByContainerId($satellite->get('container_id'), true)) {
            $this->render403();
            return;
        }

        $User = new User($this->getUser());

        $hosts = $HostsTable->getHostBySatelliteIdForDelete($satellite->get('id'));
        $hasError = false;
        if (!empty($hosts)) {
            foreach ($hosts as $host) {
                /** @var Host $host */
                if (!$this->allowedByContainerId($host->getContainerIds())) {
                    $this->render403();
                    return;
                }

                $usedBy = $host->isUsedByModules();
                if (!empty($usedBy)) {
                    $this->response = $this->response->withStatus(400);
                    $this->set('success', false);
                    $this->set('id', $satellite->get('id'));
                    $this->set('host_id', $host->get('id'));
                    $this->set('message', __('Issue while deleting host'));
                    $this->set('usedBy', $usedBy);
                    $this->viewBuilder()->setOption('serialize', ['success', 'id', 'host_id', 'message', 'usedBy']);
                    $hasError = true;
                    return;
                }

                $services = $ServicesTable->getServicesByHostIdForDelete($host->get('id'), true);
                foreach ($services as $service) {
                    /** @var Service $service */
                    $usedBy = $service->isUsedByModules();
                    if (!empty($usedBy)) {
                        $this->response = $this->response->withStatus(400);
                        $this->set('success', false);
                        $this->set('id', $satellite->get('id'));
                        $this->set('host_id', $host->get('id'));
                        $this->set('service_id', $service->get('id'));
                        $this->set('message', __('Issue while deleting host. Service used by other objects.'));
                        $this->set('usedBy', $usedBy);
                        $this->viewBuilder()->setOption('serialize', ['success', 'id', 'message', 'usedBy', 'host_id', 'service_id']);
                        $hasError = true;
                        return;
                    }
                }

                //Delete services of the host
                foreach ($services as $serviceEntity) {
                    /** @var Service $serviceEntity */
                    $service = $ServicesTable->find()
                        ->contain([
                            'ServiceescalationsServiceMemberships',
                            'ServicedependenciesServiceMemberships'
                        ])
                        ->where([
                            'Services.id' => $serviceEntity->get('id')
                        ])
                        ->firstOrFail();

                    $ServicesTable->__delete($service, $User);
                }

                //Delete the host
                if (!$HostsTable->__delete($host, $User)) {
                    $hasError = true;
                }
            }
        }

        if ($hasError) {
            $this->response = $this->response->withStatus(400);
            $this->set('id', $satellite->get('id'));
            $this->set('success', false);
            $this->set('message', __('Issue while deleting hosts...'));
            $this->viewBuilder()->setOption('serialize', ['success', 'message']);
        }

        $SatellitesTable->delete($satellite);

        $this->set('success', true);
        $this->set('message', __('Satellite successfully deleted'));
        $this->viewBuilder()->setOption('serialize', ['success', 'message']);
    }

    /****************************
     *       AJAX METHODS       *
     ****************************/

    public function name() {
        if (!$this->isAngularJsRequest()) {
            //Only ship template
            return;
        }

        $id = $this->request->getQuery('id', 0);

        /** @var $SatellitesTable SatellitesTable */
        $SatellitesTable = TableRegistry::getTableLocator()->get('DistributeModule.Satellites');

        if (!$SatellitesTable->existsById($id)) {
            throw new NotFoundException(__('Satellite not found'));
        }

        $satellite = $SatellitesTable->get($id);

        if (!$this->allowedByContainerId($satellite->get('container_id'), false)) {
            $this->render403();
            return;
        }

        $this->set('satellite', [
            'Satellite' => $satellite
        ]);
        $this->viewBuilder()->setOption('serialize', ['satellite']);
    }

    public function loadContainers() {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        /** @var $ContainersTable ContainersTable */
        $ContainersTable = TableRegistry::getTableLocator()->get('Containers');
        if ($this->hasRootPrivileges === true) {
            $containers = $ContainersTable->easyPath($this->MY_RIGHTS, OBJECT_SATELLITE, [], $this->hasRootPrivileges);
        } else {
            $containers = $ContainersTable->easyPath($this->getWriteContainers(), OBJECT_SATELLITE, [], $this->hasRootPrivileges);
        }
        $containers = Api::makeItJavaScriptAble($containers);

        $this->set('containers', $containers);
        $this->set('timezone', date_default_timezone_get());
        $this->viewBuilder()->setOption('serialize', ['containers', 'timezone']);
    }

    public function addSatelliteDowntime() {
        if (!$this->isAngularJsRequest()) {
            // ship html template
            return;
        }

        if ($this->request->is('post') || $this->request->is('put')) {
            /** @var $SystemdowntimesTable SystemdowntimesTable */
            $SystemdowntimesTable = TableRegistry::getTableLocator()->get('Systemdowntimes');
            $data = $this->request->getData('Systemdowntime');

            /** @var SatellitesTable $SatellitesTable */
            $SatellitesTable = TableRegistry::getTableLocator()->get('DistributeModule.Satellites');


            if (!isset($data['object_id']) || empty($data['object_id'])) {
                $this->response = $this->response->withStatus(400);
                $this->set('error', [
                    'object_id' => [
                        '_empty' => __('You have to select at least on object.')
                    ]
                ]);
                $this->viewBuilder()->setOption('serialize', ['error']);
                return;
            }

            if (!is_array($data['object_id'])) {
                $data['object_id'] = [$data['object_id']];
            }

            if (isset($data['weekdays']) && is_array($data['weekdays'])) {
                $data['weekdays'] = implode(',', $data['weekdays']);
            }

            $User = new User($this->getUser());

            $data['author'] = $User->getFullName();

            $objectIds = $data['object_id'];
            unset($data['object_id']);

            $Entities = [];
            foreach ($objectIds as $objectId) {
                $tmpData = $data;
                $tmpData['object_id'] = $objectId;
                $Entity = $SystemdowntimesTable->newEntity($tmpData);
                if ($Entity->hasErrors()) {
                    //On entity has an error so ALL entities has an error!
                    $this->response = $this->response->withStatus(400);
                    $this->set('error', $Entity->getErrors());
                    $this->viewBuilder()->setOption('serialize', ['error']);
                    return;
                }

                //No errors
                $Entities[] = $Entity;
            }

            $isRecurringDowntime = $data['is_recurring'] === 1 || $data['is_recurring'] === '1';
            $success = true;

            $UserTime = $User->getUserTime();

            if ($isRecurringDowntime) {
                //Recurring downtimes will get saved to the database
                foreach ($Entities as $Entity) {
                    /** @var \App\Model\Entity\Systemdowntime $Entity */
                    $Entity->shiftFromTime($UserTime);
                }

                $success = $SystemdowntimesTable->saveMany($Entities);
            } else {
                //Normal downtimes will be passed to the monitoring engine
                $GearmanClient = new Gearman();
                /** @var $ContainersTable ContainersTable */
                $ContainersTable = TableRegistry::getTableLocator()->get('Containers');
                /** @var $HostsTable HostsTable */
                $HostsTable = TableRegistry::getTableLocator()->get('Hosts');

                $offset = $UserTime->getUserTimeToServerOffset();
                foreach ($Entities as $Entity) {
                    $satellite = $SatellitesTable->get($Entity->get('object_id'));
                    $satelliteId = $satellite->get('id');
                    $containerId = $satellite->get('container_id');
                    //Lookup all satellite child containers from the user has write permissions to  select the hosts and create
                    //the downtimes

                    if ($containerId == ROOT_CONTAINER) {
                        $childrenContainers = $ContainersTable->resolveChildrenOfContainerIds(ROOT_CONTAINER, true);
                    } else {
                        $childrenContainers = $ContainersTable->resolveChildrenOfContainerIds($containerId);
                        $childrenContainers = $ContainersTable->removeRootContainer($childrenContainers);
                    }

                    if (!$this->allowedByContainerId($childrenContainers)) {
                        continue;
                    }
                    $hosts = $HostsTable->getHostBySatelliteId($satelliteId);
                    $hosts = Hash::extract($hosts, '{n}[disabled=0]');
                    if (!empty($hosts)) {
                        $start = strtotime(
                            sprintf(
                                '%s %s',
                                $Entity->get('from_date'),
                                $Entity->get('from_time')
                            ));
                        $end = strtotime(
                            sprintf('%s %s',
                                $Entity->get('to_date'),
                                $Entity->get('to_time')
                            ));

                        if (!empty($hosts)) {
                            foreach ($hosts as $host) {
                                /** @var Host $host */
                                if (!$this->allowedByContainerId($host->getContainerIds())) {
                                    continue;
                                }

                                $payload = [
                                    'hostUuid'     => $host->get('uuid'),
                                    'downtimetype' => $Entity->get('downtimetype_id'),
                                    'start'        => $start - $offset,
                                    'end'          => $end - $offset,
                                    'comment'      => $Entity->get('comment'),
                                    'author'       => $Entity->get('author'),
                                ];
                                $GearmanClient->sendBackground('createHostDowntime', $payload);
                            }
                        }
                    }
                }
            }

            $this->set('success', $success);
            $this->viewBuilder()->setOption('serialize', ['success']);
        }
    }

    public function downtime() {
        if (!$this->isAngularJsRequest()) {
            //Only ship template
            return;
        }

        $AngularRequest = new AngularRequest($this->request);
        $PaginateOMat = new PaginateOMat($this, $this->isScrollRequest(), $AngularRequest->getPage());

        $SystemdowntimesFilter = new SystemdowntimesFilter($this->request);
        $Conditions = new SystemdowntimesConditions();

        //Process conditions
        if ($this->hasRootPrivileges) {
            $Conditions->setContainerIds($this->MY_RIGHTS);
        }
        $Conditions->setOrder($AngularRequest->getOrderForPaginator('Systemdowntimes.from_time', 'desc'));
        $Conditions->setConditions($SystemdowntimesFilter->satelliteFilter());

        /** @var SatellitesTable $SatellitesTable */
        $SatellitesTable = TableRegistry::getTableLocator()->get('DistributeModule.Satellites');

        $recurringSatelliteDowntimes = $SatellitesTable->getRecurringSatelliteDowntimes($Conditions, $PaginateOMat);

        $User = new User($this->getUser());
        $UserTime = $User->getUserTime();
        //Prepare data for API
        $all_satellite_recurring_downtimes = [];
        foreach ($recurringSatelliteDowntimes as $recurringSatelliteDowntime) {
            if (!isset($recurringSatelliteDowntime['name'])) {
                continue;
            }

            if ($this->hasRootPrivileges) {
                $allowEdit = true;
            } else {
                $ContainerPermissions = new ContainerPermissions($this->MY_RIGHTS_LEVEL, [$recurringSatelliteDowntime['container_id']]);
                $allowEdit = $ContainerPermissions->hasPermission();
            }

            $Systemdowntime = new Systemdowntime($recurringSatelliteDowntime['Systemdowntimes'], $UserTime);

            $tmpRecord = [
                'Satellite'      => [
                    'name'       => $recurringSatelliteDowntime['name'],
                    'allow_edit' => $allowEdit
                ],
                'Systemdowntime' => $Systemdowntime->toArray()
            ];
            $all_satellite_recurring_downtimes[] = $tmpRecord;
        }


        $this->set('all_satellite_recurring_downtimes', $all_satellite_recurring_downtimes);
        $toJson = ['all_satellite_recurring_downtimes', 'paging'];
        if ($this->isScrollRequest()) {
            $toJson = ['all_satellite_recurring_downtimes', 'scroll'];
        }
        $this->viewBuilder()->setOption('serialize', $toJson);
    }

    public function loadHostsBySatelliteIds() {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        $satelliteIds = $this->request->getQuery('satelliteIds', []);
        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');

        $hosts = [];

        foreach($satelliteIds as $satelliteId){
            $satelliteHosts = $HostsTable->getHostBySatelliteId($satelliteId, false);
            foreach($satelliteHosts as $satelliteHost){
                $hosts[] = $satelliteHost;
            }
        }

        $this->set('hosts', $hosts);
        $this->viewBuilder()->setOption('serialize', ['hosts']);
    }
}
