<?php
declare(strict_types=1);

namespace DistributeModule\Model\Table;

use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * SatelliteTasks Model
 *
 * @method \DistributeModule\Model\Entity\SatelliteTask newEmptyEntity()
 * @method \DistributeModule\Model\Entity\SatelliteTask newEntity(array $data, array $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteTask[] newEntities(array $data, array $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteTask get($primaryKey, $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteTask findOrCreate($search, ?callable $callback = null, $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteTask patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteTask[] patchEntities(iterable $entities, array $data, array $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteTask|false save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteTask saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteTask[]|\Cake\Datasource\ResultSetInterface|false saveMany(iterable $entities, $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteTask[]|\Cake\Datasource\ResultSetInterface saveManyOrFail(iterable $entities, $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteTask[]|\Cake\Datasource\ResultSetInterface|false deleteMany(iterable $entities, $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteTask[]|\Cake\Datasource\ResultSetInterface deleteManyOrFail(iterable $entities, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 *
 * Created via:
 * oitc bake model SatelliteTasks -p DistributeModule
 */
class SatelliteTasksTable extends Table {


    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('satellite_tasks');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->belongsTo('Satellites', [
            'foreignKey' => 'satellite_id',
            'joinType'   => 'INNER',
            'className'  => 'DistributeModule.Satellites',
        ]);

        $this->addBehavior('Timestamp');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator): Validator {
        $validator
            ->integer('id')
            ->allowEmptyString('id', null, 'create');

        $validator
            ->scalar('task')
            ->maxLength('task', 255)
            ->requirePresence('task', 'create')
            ->notEmptyString('task');

        $validator
            ->integer('status')
            ->notEmptyString('status');

        $validator
            ->scalar('result')
            ->maxLength('result', 4294967295)
            ->allowEmptyString('result');

        $validator
            ->scalar('error')
            ->maxLength('error', 4294967295)
            ->allowEmptyString('error');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker {
        $rules->add($rules->existsIn(['satellite_id'], 'Satellites'));

        return $rules;
    }
}
