angular.module('openITCOCKPIT')
    .controller('SatellitesIndexController', function($scope, $http, SortService, MassChangeService){

        SortService.setSort('Satellites.name');
        SortService.setDirection('asc');
        $scope.currentPage = 1;
        $scope.useScroll = true;

        /*** Filter Settings ***/
        var defaultFilter = function(){
            $scope.filter = {
                Satellites: {
                    name: '',
                    address: '',
                    sync_method: []
                }
            };
        };
        /*** Filter end ***/
        $scope.massChange = {};
        $scope.selectedElements = 0;
        $scope.deleteUrl = '/distribute_module/satellites/delete/';

        $scope.init = true;
        $scope.showFilter = false;
        $scope.load = function(){
            $http.get("/distribute_module/satellites/index.json", {
                params: {
                    'angular': true,
                    'scroll': $scope.useScroll,
                    'sort': SortService.getSort(),
                    'page': $scope.currentPage,
                    'direction': SortService.getDirection(),
                    'filter[Satellites.name]': $scope.filter.Satellites.name,
                    'filter[Satellites.address]': $scope.filter.Satellites.address,
                    'filter[Satellites.sync_method][]': $scope.filter.Satellites.sync_method
                }
            }).then(function(result){
                $scope.satellites = result.data.all_satellites;
                $scope.paging = result.data.paging;
                $scope.scroll = result.data.scroll;
                $scope.init = false;
            });
        };

        $scope.triggerFilter = function(){
            $scope.showFilter = !$scope.showFilter === true;
        };

        $scope.resetFilter = function(){
            defaultFilter();
            $scope.undoSelection();
        };

        $scope.undoSelection = function(){
            MassChangeService.clearSelection();
            $scope.massChange = MassChangeService.getSelected();
            $scope.selectedElements = MassChangeService.getCount();
        };

        $scope.selectAll = function(){
            if($scope.satellites){
                for(var key in $scope.satellites){
                    if($scope.satellites[key].allow_edit){
                        var id = $scope.satellites[key].id;
                        $scope.massChange[id] = true;
                    }
                }
            }
        };

        $scope.getObjectsForDelete = function(){
            var objects = {};
            var selectedObjects = MassChangeService.getSelected();
            for(var key in $scope.satellites){
                for(var id in selectedObjects){
                    if(id == $scope.satellites[key].id){
                        objects[id] = $scope.satellites[key].name;
                    }

                }
            }
            return objects;
        };

        $scope.getObjectsForExternalCommand = function(){
            var selectedObjects = Object.keys(MassChangeService.getSelected());
            var objects = {};

            lastHostUuid = null;
            $http.get("/distribute_module/satellites/loadHostsBySatelliteIds.json", {
                params: {
                    'angular': true,
                    'satelliteIds[]': selectedObjects
                }
            }).then(function(result){
                    $scope.hosts = result.data.hosts;
                    for(var index in $scope.hosts){
                        objects[$scope.hosts[index].id] = {
                            'Host': $scope.hosts[index]
                        };
                    }
                    /*
                    The HostDowntimeDirective.js is sharing it's $scope with the current active AngularJS controller - for
                    this reason we can call this $scope.hostDowntime() from the link method of the HostDowntimeDirective
                     */
                    return $scope.hostDowntime(objects);
                }
            )
            ;
        };

        $scope.getObjectForDelete = function(satellite){
            var object = {};
            object[satellite.id] = satellite.name;
            return object;
        };

        $scope.changeMode = function(val){
            $scope.useScroll = val;
            $scope.load();
        };

        $scope.changepage = function(page){
            if(page !== $scope.currentPage){
                $scope.currentPage = page;
                $scope.load();
            }
        };

//Fire on page load
        defaultFilter();
        SortService.setCallback($scope.load);

        $scope.$watch('filter', function(){
            $scope.load();
        }, true);

        $scope.$watch('massChange', function(){
            MassChangeService.setSelected($scope.massChange);
            $scope.selectedElements = MassChangeService.getCount();
        }, true);
    })
;
